@@ -34,8 +34,7 @@ import math
 import statistics
 import csv
 
-from pyworkflow.object import Integer
-import pyworkflow.object as pwobj
+from pyworkflow.object import Integer, Float, String, Pointer, Boolean, CsvList
 from pwem.objects import Transform
 import pyworkflow.utils.path as path
 import pwem.objects.data as data
@@ -49,9 +48,9 @@ class TiltImageBase:
     """ Base class for TiltImageM and TiltImage. """
 
     def __init__(self, tsId=None, tiltAngle=None, acquisitionOrder=None, **kwargs):
-        self._tiltAngle = pwobj.Float(tiltAngle)
-        self._tsId = pwobj.String(tsId)
-        self._acqOrder = pwobj.Integer(acquisitionOrder)
+        self._tiltAngle = Float(tiltAngle)
+        self._tsId = String(tsId)
+        self._acqOrder = Integer(acquisitionOrder)
 
     def getTsId(self):
         """ Get unique TiltSerie ID, usually retrieved from the
@@ -111,12 +110,19 @@ class TiltImage(data.Image, TiltImageBase):
 class TiltSeriesBase(data.SetOfImages):
     def __init__(self, **kwargs):
         data.SetOfImages.__init__(self, **kwargs)
-        self._tsId = pwobj.String(kwargs.get('tsId', None))
+        self._tsId = String(kwargs.get('tsId', None))
         # TiltSeries will always be used inside a SetOfTiltSeries
         # so, let's do no store the mapper path by default
         self._mapperPath.setStore(False)
+        self._acquisition = TomoAcquisition()
+        self._origin = Transform()
+        self._anglesCount = Integer()
 
-        self._origin = None
+    def getAnglesCount(self):
+        return self._anglesCount
+
+    def setAnglesCount(self, value):
+        self._anglesCount = value
 
     def getTsId(self):
         """ Get unique TiltSerie ID, usually retrieved from the
@@ -132,6 +138,7 @@ class TiltSeriesBase(data.SetOfImages):
         not _mapperPath or _size from other set of tomograms to current one.
         """
         self.copy(other, copyId=copyId, ignoreAttrs=['_mapperPath', '_size'])
+        # self.copyAttributes(other, ['_tsId', '_anglesCount'])
 
     def append(self, tiltImage):
         tiltImage.setTsId(self.getTsId())
@@ -159,7 +166,7 @@ class TiltSeriesBase(data.SetOfImages):
 
         angleList = []
 
-        for ti in self:
+        for ti in self.iterItems(orderBy="_tiltAngle"):
             angleList.append(ti.getTiltAngle())
 
         if reverse:
@@ -200,7 +207,7 @@ class TiltSeriesBase(data.SetOfImages):
         t = Transform()
         x, y, z = self.getDim()
         if z > 1:
-            z = z / -2.
+            z /= -2.
         print(t)
         t.setShifts(x / -2. * sampling, y / -2. * sampling, z * sampling)
         return t  # The identity matrix
@@ -258,6 +265,12 @@ class TiltSeries(TiltSeriesBase):
         else:
             path.createLink(inputFilePath, outputFilePath)
 
+    def _dimStr(self):
+        """ Return the string representing the dimensions. """
+
+        return '%s x %s' % (self._firstDim[0],
+                            self._firstDim[1])
+
 
 class SetOfTiltSeriesBase(data.SetOfImages):
     EXPOSE_ITEMS = True
@@ -268,6 +281,20 @@ class SetOfTiltSeriesBase(data.SetOfImages):
 
     def __init__(self, **kwargs):
         data.SetOfImages.__init__(self, **kwargs)
+        self._anglesCount = Integer()
+        self._acquisition = TomoAcquisition()
+
+    def getAnglesCount(self):
+        return self._anglesCount.get()
+
+    def setAnglesCount(self, value):
+        self._anglesCount.set(value)
+
+    def copyInfo(self, other):
+        """ Copy information (sampling rate and ctf)
+        from other set of images to current one"""
+        super().copyInfo(other)
+        self.copyAttributes(other, '_anglesCount')
 
     def iterClassItems(self, iterDisabled=False):
         """ Iterate over the images of a class.
@@ -305,9 +332,8 @@ class SetOfTiltSeriesBase(data.SetOfImages):
         self._setItemMapperPath(classItem)
         return classItem
 
-    def iterItems(self, orderBy='id', direction='ASC'):
-        for item in data.EMSet.iterItems(self, orderBy=orderBy,
-                                         direction=direction):
+    def iterItems(self, **kwargs):
+        for item in data.EMSet.iterItems(self, **kwargs):
             self._setItemMapperPath(item)
             yield item
 
@@ -343,7 +369,9 @@ class SetOfTiltSeriesBase(data.SetOfImages):
 
     def updateDim(self):
         """ Update dimensions of this set base on the first element. """
-        self.setDim(self.getFirstItem().getDim())
+        firstItem = self.getFirstItem()
+        self.setDim(firstItem.getDim())
+        self._anglesCount.set(firstItem.getSize())
 
     def getScannedPixelSize(self):
         mag = self._acquisition.getMagnification()
@@ -353,6 +381,12 @@ class SetOfTiltSeriesBase(data.SetOfImages):
 class SetOfTiltSeries(SetOfTiltSeriesBase):
     ITEM_TYPE = TiltSeries
 
+    def _dimStr(self):
+        """ Return the string representing the dimensions. """
+
+        return '%s x %s x %s' % (self._anglesCount, self._firstDim[0],
+                                 self._firstDim[1])
+
 
 class TiltImageM(data.Movie, TiltImageBase):
     """ Tilt movie. """
@@ -375,8 +409,8 @@ class SetOfTiltSeriesM(SetOfTiltSeriesBase):
 
     def __init__(self, **kwargs):
         SetOfTiltSeriesBase.__init__(self, **kwargs)
-        self._gainFile = pwobj.String()
-        self._darkFile = pwobj.String()
+        self._gainFile = String()
+        self._darkFile = String()
         # Store the frames range to avoid loading the items
         self._firstFramesRange = data.FramesRange()
 
@@ -405,6 +439,11 @@ class SetOfTiltSeriesM(SetOfTiltSeriesBase):
         self._darkFile.set(other.getDark())
         # self._firstFramesRange.set(other.getFramesRange())
 
+    def _dimStr(self):
+        """ Return the string representing the dimensions. """
+
+        return '%s x %s' % (self._anglesCount, self._firstDim)
+
 
 class TiltSeriesDict:
     """ Helper class that to store TiltSeries and TiltImage but
@@ -535,13 +574,22 @@ class TiltSeriesDict:
 
 
 class TomoAcquisition(data.Acquisition):
-    def __init__(self, angleMin=None, angleMax=None, step=None, angleAxis1=None, angleAxis2=None, **kwargs):
+    def __init__(self, angleMin=None, angleMax=None, step=None, angleAxis1=None,
+                 angleAxis2=None, accumDose=None, tiltAxisAngle=None, **kwargs):
         data.Acquisition.__init__(self, **kwargs)
-        self._angleMin = pwobj.Float(angleMin)
-        self._angleMax = pwobj.Float(angleMax)
-        self._step = pwobj.Float(step)
-        self._angleAxis1 = pwobj.Float(angleAxis1)
-        self._angleAxis2 = pwobj.Float(angleAxis2)
+        self._angleMin = Float(angleMin)
+        self._angleMax = Float(angleMax)
+        self._step = Float(step)
+        self._angleAxis1 = Float(angleAxis1)
+        self._angleAxis2 = Float(angleAxis2)
+        self._accumDose = Float(accumDose)
+        self._tiltAxisAngle = Float(tiltAxisAngle)
+
+    def getTiltAxisAngle(self):
+        return self._tiltAxisAngle.get()
+
+    def setTiltAxisAngle(self, value):
+        self._tiltAxisAngle.set(value)
 
     def getAngleMax(self):
         return self._angleMax.get()
@@ -573,6 +621,12 @@ class TomoAcquisition(data.Acquisition):
     def setAngleAxis2(self, value):
         self._angleAxis2.set(value)
 
+    def getAccumDose(self):
+        return self._accumDose.get()
+
+    def setAccumDose(self, value):
+        self._accumDose.set(value)
+
 
 class Tomogram(data.Volume):
     """ Class to hold the tomogram abstraction inside Scipion. The origin (self._origin) of the volume is set as the
@@ -583,7 +637,7 @@ class Tomogram(data.Volume):
     def __init__(self, **kwargs):
         data.Volume.__init__(self, **kwargs)
         self._acquisition = None
-        self._tsId = pwobj.String(kwargs.get('tsId', None))
+        self._tsId = String(kwargs.get('tsId', None))
         self._dim = None
 
     def getTsId(self):
@@ -628,7 +682,6 @@ class Tomogram(data.Volume):
         return None
 
 
-
 class SetOfTomograms(data.SetOfVolumes):
     ITEM_TYPE = Tomogram
     EXPOSE_ITEMS = True
@@ -647,7 +700,7 @@ class TomoMask(Tomogram):
     """
     def __init__(self, **kwargs):
         Tomogram.__init__(self, **kwargs)
-        self._volName = pwobj.String()
+        self._volName = String()
 
     def getVolName(self):
         """ Get the reference tomogram file for the current tomoMask.
@@ -680,25 +733,25 @@ class Coordinate3D(data.EMObject):
 
     def __init__(self, **kwargs):
         data.EMObject.__init__(self, **kwargs)
-        self._volumePointer = pwobj.Pointer(objDoStore=False)
-        self._x = Integer()
-        self._y = Integer()
-        self._z = Integer()
-        self._volId = pwobj.Integer()
+        self._volumePointer = Pointer(objDoStore=False)
+        self._x = Float()
+        self._y = Float()
+        self._z = Float()
+        self._volId = Integer()
         self._eulerMatrix = data.Transform()
-        self._groupId = pwobj.Integer()  # This may refer to a mesh, ROI, vesicle or any group of coordinates
+        self._groupId = Integer()  # This may refer to a mesh, ROI, vesicle or any group of coordinates
 
     def getX(self, originFunction):
-        ''' See getPosition method for a full description of how "originFunction"
-        works'''
+        """ See getPosition method for a full description of how "originFunction"
+        works"""
         origin_Scipion = self.getVolumeOrigin()[0]
         aux = originFunction(self.getVolume().getDim())
         origin = aux[0] if aux is not None else -origin_Scipion
         return self._x.get() - origin - origin_Scipion
 
     def setX(self, x, originFunction):
-        ''' See setPosition method for a full description of how "originFunction"
-        works'''
+        """ See setPosition method for a full description of how "originFunction"
+        works"""
         origin_Scipion = self.getVolumeOrigin()[0]
         aux = originFunction(self.getVolume().getDim())
         origin = aux[0] if aux is not None else -origin_Scipion
@@ -708,16 +761,16 @@ class Coordinate3D(data.EMObject):
         self._x.sum(shiftX)
 
     def getY(self, originFunction):
-        ''' See getPosition method for a full description of how "originFunction"
-        works'''
+        """ See getPosition method for a full description of how "originFunction"
+        works"""
         origin_Scipion = self.getVolumeOrigin()[1]
         aux = originFunction(self.getVolume().getDim())
         origin = aux[1] if aux is not None else -origin_Scipion
         return self._y.get() - origin - origin_Scipion
 
     def setY(self, y, originFunction):
-        ''' See setPosition method for a full description of how "originFunction"
-        works'''
+        """ See setPosition method for a full description of how "originFunction"
+        works"""
         origin_Scipion = self.getVolumeOrigin()[1]
         aux = originFunction(self.getVolume().getDim())
         origin = aux[1] if aux is not None else -origin_Scipion
@@ -727,16 +780,16 @@ class Coordinate3D(data.EMObject):
         self._y.sum(shiftY)
 
     def getZ(self, originFunction):
-        ''' See getPosition method for a full description of how "originFunction"
-        works'''
+        """ See getPosition method for a full description of how "originFunction"
+        works"""
         origin_Scipion = self.getVolumeOrigin()[2]
         aux = originFunction(self.getVolume().getDim())
         origin = aux[2] if aux is not None else -origin_Scipion
         return self._z.get() - origin - origin_Scipion
 
     def setZ(self, z, originFunction):
-        ''' See setPosition method for a full description of how "originFunction"
-        works'''
+        """ See setPosition method for a full description of how "originFunction"
+        works"""
         origin_Scipion = self.getVolumeOrigin()[2]
         aux = originFunction(self.getVolume().getDim())
         origin = aux[2] if aux is not None else -origin_Scipion
@@ -781,7 +834,7 @@ class Coordinate3D(data.EMObject):
         self._z.multiply(factor)
 
     def getPosition(self, originFunction):
-        '''Get the position a Coordinate3D refered to a given origin defined by originFunction.
+        """Get the position a Coordinate3D refered to a given origin defined by originFunction.
         The input of the method is a funtion (originFunction) which moves the coordinate
         position refered to the bottom left corner to other origin (retrieved by originFunction) in the grid.
 
@@ -797,7 +850,7 @@ class Coordinate3D(data.EMObject):
 
         Firstly, the Scipion origin vector stored in the Tomogram associated to the Coordinate3D
         will be applied to refer the current coordinate to the bottom left coordinate of the Tomogram.
-        '''
+        """
         return self.getX(originFunction), self.getY(originFunction), self.getZ(originFunction)
 
     def setPosition(self, x, y, z, originFunction):
@@ -892,17 +945,19 @@ class Coordinate3D(data.EMObject):
         return self._groupId is not None
 
     def getVolumeOrigin(self, angstrom=False):
-        '''Return the a vector that can be used to move the position of the Coordinate3D
+        """Return the a vector that can be used to move the position of the Coordinate3D
         (referred to the center of the Tomogram or other origin specified by the user)
         to the bottom left corner of the Tomogram
-        '''
+        """
+        vol = self.getVolume()
+        if not vol:
+            raise Exception("3D coordinate must be referred to a volume to get its origin.")
         if angstrom:
-            return self.getVolume().getShiftsFromOrigin()
+            return vol.getShiftsFromOrigin()
         else:
-            vol = self.getVolume()
             sr = vol.getSamplingRate()
             origin = vol.getShiftsFromOrigin()
-            return int(origin[0] / sr), int(origin[1] / sr), int(origin[2] / sr)
+            return origin[0] / sr, origin[1] / sr, origin[2] / sr
 
 
 class SetOfCoordinates3D(data.EMSet):
@@ -914,9 +969,9 @@ class SetOfCoordinates3D(data.EMSet):
 
     def __init__(self, **kwargs):
         data.EMSet.__init__(self, **kwargs)
-        self._boxSize = pwobj.Integer()
-        self._samplingRate = pwobj.Float()
-        self._precedentsPointer = pwobj.Pointer()
+        self._boxSize = Integer()
+        self._samplingRate = Float()
+        self._precedentsPointer = Pointer()
 
     def getBoxSize(self):
         """ Return the box size of the particles.
@@ -1038,7 +1093,7 @@ class SetOfCoordinates3D(data.EMSet):
         return coord
 
     def __getitem__(self, itemId):
-        '''Add a pointer to a Tomogram before returning the Coordinate3D'''
+        """Add a pointer to a Tomogram before returning the Coordinate3D"""
         coord = data.EMSet.__getitem__(self, itemId)
         # In case pointer is lost in a for loop
         # clone = self.getPrecedents().getClass()()
@@ -1054,7 +1109,7 @@ class SubTomogram(data.Volume):
         self._acquisition = None
         self._coordinate = None
         self._volId = Integer()
-        self._volName = pwobj.String()
+        self._volName = String()
 
     def hasCoordinate3D(self):
         return self._coordinate is not None
@@ -1063,10 +1118,12 @@ class SubTomogram(data.Volume):
         self._coordinate = coordinate
 
     def getCoordinate3D(self):
-        '''Since the object Coordinate3D needs a volume, use the information stored in the
-        SubTomogram to reconstruct the corresponding Tomogram associated to its Coordinate3D'''
+        """Since the object Coordinate3D needs a volume, use the information stored in the
+        SubTomogram to reconstruct the corresponding Tomogram associated to its Coordinate3D"""
         tomo = Tomogram()
-        tomo.setOrigin(self.getOrigin())
+        subtomoOrigin = self.getOrigin()
+        if subtomoOrigin:
+            tomo.setOrigin(subtomoOrigin)
         tomo.setLocation(self.getVolName())
         tomo.setSamplingRate(self.getSamplingRate())
         coord = self._coordinate
@@ -1112,10 +1169,10 @@ class SubTomogram(data.Volume):
         self._volName.set(volName)
 
     def getVolumeOrigin(self, angstrom=False):
-        '''Return the a vector that can be used to move the position of the Coordinate3D
+        """Return the a vector that can be used to move the position of the Coordinate3D
         associated to the SubTomogram (referred to the center of the Tomogram or other
         origin specified by the user) to the bottom left corner of the Tomogram
-        '''
+        """
         if angstrom:
             return self.getShiftsFromOrigin()
         else:
@@ -1131,7 +1188,7 @@ class SetOfSubTomograms(data.SetOfVolumes):
     def __init__(self, **kwargs):
         data.SetOfVolumes.__init__(self, **kwargs)
         self._acquisition = TomoAcquisition()
-        self._coordsPointer = pwobj.Pointer()
+        self._coordsPointer = Pointer()
 
     def hasCoordinates3D(self):
         return self._coordsPointer.hasValue()
@@ -1199,9 +1256,27 @@ class LandmarkModel(data.EMObject):
 
     def __init__(self, tsId=None, fileName=None, modelName=None, **kwargs):
         data.EMObject.__init__(self, **kwargs)
-        self._tsId = pwobj.String(tsId)
-        self._fileName = pwobj.String(fileName)
-        self._modelName = pwobj.String(modelName)
+        self._tsId = String(tsId)
+        self._fileName = String(fileName)
+        self._modelName = String(modelName)
+        self._tiltSeries = Pointer(objDoStore=False)
+
+
+    def getTiltSeries(self):
+        """ Return the tilt-series associated with this landmark model. """
+
+        return self._tiltSeries.get()
+
+    def setTiltSeries(self, tiltSeries):
+        """ Set the tilt-series from which this landmark model were calculated.
+        :param tiltSeries: Either a TiltSeries object or a pointer to it.
+        """
+
+        if tiltSeries.isPointer():
+            self._tiltSeries.copy(tiltSeries)
+
+        else:
+            self._tiltSeries.set(tiltSeries)
 
     def getTsId(self):
         return str(self._tsId)
@@ -1213,13 +1288,13 @@ class LandmarkModel(data.EMObject):
         return str(self._modelName)
 
     def setTsId(self, tsId):
-        self._tsId = pwobj.String(tsId)
+        self._tsId = String(tsId)
 
     def setFileName(self, fileName):
-        self._fileName = pwobj.String(fileName)
+        self._fileName = String(fileName)
 
     def setModelName(self, modelName):
-        self._modelName = pwobj.String(modelName)
+        self._modelName = String(modelName)
 
     def addLandmark(self, xCoor, yCoor, tiltIm, chainId, xResid, yResid):
         fieldNames = ['xCoor', 'yCoor', 'tiltIm', 'chainId', 'xResid', 'yResid']
@@ -1263,7 +1338,58 @@ class SetOfLandmarkModels(data.EMSet):
     ITEM_TYPE = LandmarkModel
 
     def __init__(self, **kwargs):
-        data.EMSet.__init__(self, **kwargs)
+        super().__init__(**kwargs)
+        self._setOfTiltSeriesPointer = Pointer()
+
+    def __getitem__(self, itemId):
+        """Add a pointer to a tilt-series before returning the landmark model"""
+
+        lm = super().__getitem__(itemId)
+
+        return self.completeLandmarkModel(lm)
+
+    def completeLandmarkModel(self, lm):
+        """This method completes a landmark model object setting in execution time the tilt-series associated to it,
+        since it is not possible to save pointers in the item classes of the set.
+
+        IMPORTANT: this method must be implement every time it is necesary to retrive information from the tilt-series
+        associated to the landmark models that compose the set."""
+
+        tsId = lm.getTsId()
+
+        # Check for tilt series in set with coincident tsId
+        for ts in self.getSetOfTiltSeries().iterItems(where="_tsId=='%s'" % tsId):
+            lm.setTiltSeries(ts)
+
+        return lm
+
+    def getLandmarkModelFromTsId(self, tsId):
+        """ This method return the landmark model belonging to the set that has a coincident input tsId.
+
+        :param tsId: tilt-series ID to search the landmark model into the set."""
+
+        for lm in self.iterItems(where="_tsId=='%s'" % tsId):
+            return lm
+
+    def getSetOfTiltSeries(self, pointer = False):
+        """ Return the set of tilt-series associated with this set of landmark models. """
+
+        if pointer:
+            return self._setOfTiltSeriesPointer
+
+        else:
+            return self._setOfTiltSeriesPointer.get()
+
+    def setSetOfTiltSeries(self, setOfTiltSeries):
+        """ Set the set of tilt-series from which this set of landmark models were calculted.
+        :param tiltSeries: Either a TiltSeries object or a pointer to it.
+        """
+
+        if setOfTiltSeries.isPointer():
+            self._setOfTiltSeriesPointer.copy(setOfTiltSeries)
+
+        else:
+            self._setOfTiltSeriesPointer.set(setOfTiltSeries)
 
 
 class MeshPoint(Coordinate3D):
@@ -1274,7 +1400,7 @@ class MeshPoint(Coordinate3D):
 
     def __init__(self, **kwargs):
         Coordinate3D.__init__(self, **kwargs)
-        self._volumeName = pwobj.String()
+        self._volumeName = String()
         self._description = None  # Algebraic description of fitted mesh
 
     def getVolumeName(self):
@@ -1299,7 +1425,7 @@ class SetOfMeshes(SetOfCoordinates3D):
 
     def __init__(self, **kwargs):
         SetOfCoordinates3D.__init__(self, **kwargs)
-        self._numberOfMeshes = pwobj.Integer()  # Indicates how many meshes are in the set
+        self._numberOfMeshes = Integer()  # Indicates how many meshes are in the set
 
     def getNumberOfMeshes(self):
         return self._numberOfMeshes.get()
@@ -1313,9 +1439,9 @@ class Ellipsoid(data.EMObject):
 
     def __init__(self, **kwargs):
         data.EMObject.__init__(self, **kwargs)
-        self._center = pwobj.String()
-        self._radii = pwobj.String()
-        self._algebraicDesc = pwobj.String()
+        self._center = String()
+        self._radii = String()
+        self._algebraicDesc = String()
 
     def getCenter(self):
         return self._center.get()
@@ -1344,27 +1470,78 @@ class CTFTomo(data.CTFModel):
 
     def __init__(self, **kwargs):
         data.CTFModel.__init__(self, **kwargs)
-        self._index = pwobj.Integer(kwargs.get('index', None))
-        self._defocusUDeviation = pwobj.Float()
-        self._isDefocusUDeviationInRange = pwobj.Boolean(True)
-        self._defocusVDeviation = pwobj.Float()
-        self._isDefocusVDeviationInRange = pwobj.Boolean(True)
+        self._index = Integer(kwargs.get('index', None))
+        self._defocusUDeviation = Float()
+        self._isDefocusUDeviationInRange = Boolean(True)
+        self._defocusVDeviation = Float()
+        self._isDefocusVDeviationInRange = Boolean(True)
+
+    def copyInfo(self, other, copyId=False):
+        self.copyAttributes(other, '_defocusU', '_defocusV', '_defocusAngle', '_defocusRatio', '_psdFile',
+                            '_resolution', '_fitQuality', '_index', '_defocusUDeviation', '_defocusVDeviation')
+
+        self.setEnabled(other.isEnabled())
+
+        self.setIsDefocusUDeviationInRange(other.getIsDefocusUDeviationInRange())
+        self.setIsDefocusVDeviationInRange(other.getIsDefocusVDeviationInRange())
+
+        if other.hasPhaseShift():
+            self.setPhaseShift(other.getPhaseShift())
+
+        if other.hasEstimationInfoAsList():
+            if other.hasAstigmatismInfoAsList():
+                self._defocusUList = CsvList(pType=float)
+                self._defocusVList = CsvList(pType=float)
+                self._defocusAngleList = CsvList(pType=float)
+                self.setDefocusUList(other.getDefocusUList())
+                self.setDefocusVList(other.getDefocusVList())
+                self.setDefocusAngleList(other.getDefocusAngleList())
+
+            else:
+                self._defocusUList = CsvList(pType=float)
+                self.setDefocusUList(other.getDefocusUList())
+
+        if other.hasPhaseShiftInfoAsList():
+            self._phaseShiftList = CsvList(pType=float)
+            self.setPhaseShiftList(other.getPhaseShiftList())
+
+        if other.hasCutOnFrequncyInfoAsList():
+            self._cutOnFreqList = CsvList(pType=float)
+            self.setCutOnFreqList(other.getCutOnFreqList())
+            self.setCutOnFreq(other.getCutOnFreq())
+
+        if copyId:
+            self.copyObjId(other)
+
+    @staticmethod
+    def ctfModelToCtfTomo(ctfModel):
+        newCTFTomo = CTFTomo()
+        newCTFTomo.copyAttributes(ctfModel, '_defocusU', '_defocusV',
+                                  '_defocusAngle', '_defocusRatio', '_psdFile',
+                                  '_resolution',  '_fitQuality')
+        return newCTFTomo
 
     def getIndex(self):
         return self._index
 
     def setIndex(self, value):
-        self._index = pwobj.Integer(value)
+        self._index = Integer(value)
 
     def getdefocusUDeviation(self):
         return self._defocusUDeviation
 
+    def setIsDefocusUDeviationInRange(self, value):
+        self._isDefocusUDeviationInRange = Boolean(value)
+
     def getIsDefocusUDeviationInRange(self):
         return self._isDefocusUDeviationInRange
 
     def getdefocusVDeviation(self):
         return self._defocusVDeviation
 
+    def setIsDefocusVDeviationInRange(self, value):
+        self._isDefocusVDeviationInRange = Boolean(value)
+
     def getIsDefocusVDeviationInRange(self):
         return self._isDefocusVDeviationInRange
 
@@ -1372,7 +1549,7 @@ class CTFTomo(data.CTFModel):
         return self._cutOnFreq
 
     def setCutOnFreq(self, value):
-        self._cutOnFreq = pwobj.Float(value)
+        self._cutOnFreq = Float(value)
 
     " List data methods allow compatibility with IMOD metadata. "
 
@@ -1437,7 +1614,21 @@ class CTFTomo(data.CTFModel):
         else:
             return False
 
-    # TODO: cut on frequency
+    def hasPhaseShiftInfoAsList(self):
+        """ This method checks if the CTFTomo object contains phase shift information in the form of a list. """
+
+        if hasattr(self, "_phaseShiftList"):
+            return True
+        else:
+            return False
+
+    def hasCutOnFrequncyInfoAsList(self):
+        """ This method checks if the CTFTomo object contains cut-on frequency information in the form of a list. """
+
+        if hasattr(self, "_cutOnFreqList"):
+            return True
+        else:
+            return False
 
     def completeInfoFromList(self):
         """ This method will set the _defocusU, _defocusV and _defocusAngle attributes from the provided CTF estimation
@@ -1614,10 +1805,10 @@ class CTFTomoSeries(data.EMSet):
 
     def __init__(self, **kwargs):
         data.EMSet.__init__(self, **kwargs)
-        self._tiltSeriesPointer = pwobj.Pointer(kwargs.get('tiltSeriesPointer', None))
-        self._tsId = pwobj.String(kwargs.get('tsId', None))
-        self._isDefocusUDeviationInRange = pwobj.Boolean(True)
-        self._isDefocusVDeviationInRange = pwobj.Boolean(True)
+        self._tiltSeriesPointer = Pointer(kwargs.get('tiltSeriesPointer', None))
+        self._tsId = String(kwargs.get('tsId', None))
+        self._isDefocusUDeviationInRange = Boolean(True)
+        self._isDefocusVDeviationInRange = Boolean(True)
 
         # CtfModels will always be used inside a SetOfTiltSeries
         # so, let's do no store the mapper path by default
@@ -1662,7 +1853,7 @@ class CTFTomoSeries(data.EMSet):
         """ Set the tilt-images range size used for estimation.
         :param estimationRange: Integer of the range size. """
 
-        self._estimationsInRange = pwobj.Integer(estimationRange)
+        self._estimationsInRange = Integer(estimationRange)
 
     def getIMODDefocusFileFlag(self):
         """ Return the format file from which the CTF estimation information has been acquired. This parameter is
@@ -1701,7 +1892,7 @@ class CTFTomoSeries(data.EMSet):
 
              from https://bio3d.colorado.edu/imod/doc/man/ctfphaseflip.html """
 
-        self._IMODDefocusFileFlag = pwobj.Integer(flag)
+        self._IMODDefocusFileFlag = Integer(flag)
 
     def setNumberOfEstimationsInRangeFromDefocusList(self):
         """ Set the tilt-images estimation range size used for estimation from the defocus info list size. """
@@ -1728,9 +1919,15 @@ class CTFTomoSeries(data.EMSet):
     def getIsDefocusUDeviationInRange(self):
         return self._isDefocusUDeviationInRange
 
+    def setIsDefocusUDeviationInRange(self, value):
+        self._isDefocusUDeviationInRange = Boolean(value)
+
     def getIsDefocusVDeviationInRange(self):
         return self._isDefocusVDeviationInRange
 
+    def setIsDefocusVDeviationInRange(self, value):
+        self._isDefocusVDeviationInRange = Boolean(value)
+
     def calculateDefocusUDeviation(self, defocusUTolerance=20):
         defocusUValueList = []
         for ctfTomo in self:
@@ -1741,7 +1938,7 @@ class CTFTomoSeries(data.EMSet):
         for ctfTomo in self.iterItems(iterate=False):
             ctfTomo._defocusUDeviation.set(ctfTomo.getDefocusUDeviation(mean))
             isDefocusUDeviationInRange = ctfTomo.isDefocusUDeviationInRange(mean,
-                                                    percentage=defocusUTolerance)
+                                                                            percentage=defocusUTolerance)
             if not isDefocusUDeviationInRange:
                 self._isDefocusUDeviationInRange.set(False)
             ctfTomo._isDefocusUDeviationInRange.set(isDefocusUDeviationInRange)
@@ -1757,7 +1954,7 @@ class CTFTomoSeries(data.EMSet):
         for ctfTomo in self.iterItems(iterate=False):
             ctfTomo._defocusVDeviation.set(ctfTomo.getDefocusVDeviation(mean))
             isDefocusVDeviationInRange = ctfTomo.isDefocusVDeviationInRange(mean,
-                                                    percentage=defocusVTolerance)
+                                                                            percentage=defocusVTolerance)
             if not isDefocusVDeviationInRange:
                 self._isDefocusVDeviationInRange.set(False)
             ctfTomo._isDefocusVDeviationInRange.set(isDefocusVDeviationInRange)
@@ -1771,15 +1968,15 @@ class SetOfCTFTomoSeries(data.EMSet):
 
     def __init__(self, **kwargs):
         data.EMSet.__init__(self, **kwargs)
-        self._setOfTiltSeriesPointer = pwobj.Pointer(kwargs.get('tiltSeriesPointer', None))
+        self._setOfTiltSeriesPointer = Pointer(kwargs.get('tiltSeriesPointer', None))
 
     def copyInfo(self, other):
         data.EMSet.copyInfo(self, other)
-        self._setOfTiltSeriesPointer.set(other.getSetOfTiltSeries())
+        self.setSetOfTiltSeries(other.getSetOfTiltSeries(pointer=True))
 
-    def getSetOfTiltSeries(self):
+    def getSetOfTiltSeries(self, pointer=False):
         """ Return the tilt-series associated with this CTF model series. """
-        return self._setOfTiltSeriesPointer.get()
+        return self._setOfTiltSeriesPointer.get() if not pointer else self._setOfTiltSeriesPointer
 
     def setSetOfTiltSeries(self, setOfTiltSeries):
         """ Set the tilt-series from which this CTF model series were estimated.
@@ -1820,7 +2017,7 @@ class SetOfCTFTomoSeries(data.EMSet):
         classItem = data.EMSet.__getitem__(self, itemId)
 
         objId = None
-        for tiltSeries in self.getSetOfTiltSeries():
+        for tiltSeries in self.getSetOfTiltSeries().iterItems(iterate=False):
             if tiltSeries.getTsId() == classItem.getTsId():
                 objId = tiltSeries.getObjId()
 
