@@ -27,10 +27,12 @@ import glob
 import os
 from os.path import join
 
+import numpy
 from pyworkflow.tests import BaseTest, setupTestProject
 from tomo.protocols.protocol_ts_import import MDoc
 
 from . import DataSet
+from ..constants import BOTTOM_LEFT_CORNER
 from ..utils import existsPlugin
 import tomo.protocols
 
@@ -38,18 +40,18 @@ import tomo.protocols
 class TestTomoImportSubTomograms(BaseTest):
     """ This class check if the protocol to import sub tomograms works
      properly."""
+
     @classmethod
     def setUpClass(cls):
-         setupTestProject(cls)
-         cls.dataset = DataSet.getDataSet('tomo-em')
-         # cls.tomogram = cls.dataset.getFile('tomo1')
-         # cls.coords3D = cls.dataset.getFile('overview_wbp.txt')
-         cls.table = cls.dataset.getFile('initial.tbl')
-         cls.path = cls.dataset.getPath()
-         cls.subtomos = cls.dataset.getFile('basename.hdf')
+        setupTestProject(cls)
+        cls.dataset = DataSet.getDataSet('tomo-em')
+        # cls.tomogram = cls.dataset.getFile('tomo1')
+        # cls.coords3D = cls.dataset.getFile('overview_wbp.txt')
+        cls.table = cls.dataset.getFile('initial.tbl')
+        cls.path = cls.dataset.getPath()
+        cls.subtomos = cls.dataset.getFile('basename.hdf')
 
     def _runImportSubTomograms(self):
-
         # protImportTomogram = self.newProtocol(tomo.protocols.ProtImportTomograms,
         #                                       filesPath=self.tomogram,
         #                                       samplingRate=5)
@@ -66,12 +68,11 @@ class TestTomoImportSubTomograms(BaseTest):
         protImport = self.newProtocol(tomo.protocols.ProtImportSubTomograms,
                                       filesPath=self.subtomos,
                                       samplingRate=1.35)
-                                      # importCoordinates=protImportCoordinates3d.outputCoordinates)
+        # importCoordinates=protImportCoordinates3d.outputCoordinates)
         self.launchProtocol(protImport)
         return protImport
 
     def _runImportSubTomograms2(self):
-
         # protImportTomogram = self.newProtocol(tomo.protocols.ProtImportTomograms,
         #                                       filesPath=self.tomogram,
         #                                       filesPattern='',
@@ -88,44 +89,51 @@ class TestTomoImportSubTomograms(BaseTest):
 
         protImport = self.newProtocol(tomo.protocols.ProtImportSubTomograms,
                                       filesPath=self.subtomos,
-                                      samplingRate=1.35)
-                                      # importCoordinates=protImportCoordinates3d.outputCoordinates)
+                                      samplingRate=1.35,
+                                      acquisitionAngleMax=40,
+                                      acquisitionAngleMin=-40)
+        # importCoordinates=protImportCoordinates3d.outputCoordinates)
         self.launchProtocol(protImport)
         return protImport
 
     def test_import_sub_tomograms(self):
-         protImport = self._runImportSubTomograms()
-         output = getattr(protImport, 'outputSubTomograms', None)
-         self.assertTrue(output.getSamplingRate() == 1.35)
-         self.assertTrue(output.getFirstItem().getSamplingRate() == 1.35)
-         self.assertTrue(output.getDim()[0] == 32)
-         self.assertTrue(output.getDim()[1] == 32)
-         self.assertTrue(output.getDim()[2] == 32)
-         # self.assertTrue(output.getFirstItem().getCoordinate3D().getX() == 314)
-         # self.assertTrue(output.getFirstItem().getCoordinate3D().getY() == 350)
-         # self.assertTrue(output.getFirstItem().getCoordinate3D().getZ() == 256)
-         self.assertIsNotNone(output,
+        protImport = self._runImportSubTomograms()
+        output = getattr(protImport, 'outputSubTomograms', None)
+        self.assertTrue(output.getSamplingRate() == 1.35)
+        self.assertTrue(output.getFirstItem().getSamplingRate() == 1.35)
+        self.assertTrue(output.getDim()[0] == 32)
+        self.assertTrue(output.getDim()[1] == 32)
+        self.assertTrue(output.getDim()[2] == 32)
+        self.assertEqual(output.getFirstItem().getAcquisition().getAngleMax(), 60)
+        self.assertEqual(output.getFirstItem().getAcquisition().getAngleMin(), -60)
+        # self.assertTrue(output.getFirstItem().getCoordinate3D().getX() == 314)
+        # self.assertTrue(output.getFirstItem().getCoordinate3D().getY() == 350)
+        # self.assertTrue(output.getFirstItem().getCoordinate3D().getZ() == 256)
+        self.assertIsNotNone(output,
                              "There was a problem with Import SubTomograms protocol")
 
-         protImport2 = self._runImportSubTomograms2()
-         output2 = getattr(protImport2, 'outputSubTomograms', None)
-         self.assertIsNotNone(output2,
-                              "There was a problem with Import SubTomograms protocol")
-         self.assertTrue(output2.getSamplingRate() == 1.35)
-         self.assertTrue(output2.getDim()[0] == 32)
-         self.assertTrue(output2.getDim()[1] == 32)
-         self.assertTrue(output2.getDim()[2] == 32)
-         # for i, subtomo in enumerate(output2.iterItems()):
-         #     if i == 1:
-         #         self.assertTrue(subtomo.getCoordinate3D().getX() == 174)
-         #         self.assertTrue(subtomo.getCoordinate3D().getY() == 172)
-         #         self.assertTrue(subtomo.getCoordinate3D().getZ() == 256)
-         #     if i == 0:
-         #         self.assertTrue(subtomo.getCoordinate3D().getX() == 314)
-         #         self.assertTrue(subtomo.getCoordinate3D().getY() == 350)
-         #         self.assertTrue(subtomo.getCoordinate3D().getZ() == 256)
+        protImport2 = self._runImportSubTomograms2()
+        output2 = getattr(protImport2, 'outputSubTomograms', None)
+        self.assertIsNotNone(output2,
+                             "There was a problem with Import SubTomograms protocol")
+        self.assertTrue(output2.getSamplingRate() == 1.35)
+        self.assertTrue(output2.getDim()[0] == 32)
+        self.assertTrue(output2.getDim()[1] == 32)
+        self.assertTrue(output2.getDim()[2] == 32)
+        self.assertEqual(output2.getFirstItem().getAcquisition().getAngleMax(), 40)
+        self.assertEqual(output2.getFirstItem().getAcquisition().getAngleMin(), -40)
+        # for i, subtomo in enumerate(output2.iterItems()):
+        #     if i == 1:
+        #         self.assertTrue(subtomo.getCoordinate3D().getX() == 174)
+        #         self.assertTrue(subtomo.getCoordinate3D().getY() == 172)
+        #         self.assertTrue(subtomo.getCoordinate3D().getZ() == 256)
+        #     if i == 0:
+        #         self.assertTrue(subtomo.getCoordinate3D().getX() == 314)
+        #         self.assertTrue(subtomo.getCoordinate3D().getY() == 350)
+        #         self.assertTrue(subtomo.getCoordinate3D().getZ() == 256)
+
+        return output2
 
-         return output2
 
 class TestTomoImportSetOfCoordinates3D(BaseTest):
     """This class check if the protocol to import set of coordinates 3d works properly."""
@@ -149,12 +157,12 @@ class TestTomoImportSetOfCoordinates3D(BaseTest):
                              "There was a problem with tomogram output")
 
         protImportCoordinates3d = self.newProtocol(tomo.protocols.ProtImportCoordinates3D,
-                                 objLabel='Import from %s - %s' %(program, ext),
-                                 auto=tomo.protocols.ProtImportCoordinates3D.IMPORT_FROM_AUTO,
-                                 filesPath=self.dataset.getPath(),
-                                 importTomograms=protImportTomogram.outputTomograms,
-                                 filesPattern=pattern, boxSize=32,
-                                 samplingRate=5.5)
+                                                   objLabel='Import from %s - %s' % (program, ext),
+                                                   auto=tomo.protocols.ProtImportCoordinates3D.IMPORT_FROM_AUTO,
+                                                   filesPath=self.dataset.getPath(),
+                                                   importTomograms=protImportTomogram.outputTomograms,
+                                                   filesPattern=pattern, boxSize=32,
+                                                   samplingRate=5.5)
         self.launchProtocol(protImportCoordinates3d)
 
         return protImportCoordinates3d
@@ -170,6 +178,15 @@ class TestTomoImportSetOfCoordinates3D(BaseTest):
             output = getattr(protCoordinates, 'outputCoordinates', None)
             self.assertCoordinates(output, 19)
 
+            # Check se are not loosing precision
+            firstCoord = output.getFirstItem()
+            # First row --> 224, 316, 260
+            emanCoords = [224, 316, 260]
+            self.assertEqual(firstCoord.getX(BOTTOM_LEFT_CORNER), emanCoords[0], "eman coordinate x has a wrong value")
+            self.assertEqual(firstCoord.getY(BOTTOM_LEFT_CORNER), emanCoords[1], "eman coordinate y has a wrong value")
+            self.assertEqual(firstCoord.getZ(BOTTOM_LEFT_CORNER), emanCoords[2], "eman coordinate z has a wrong value")
+
+
         if existsPlugin('dynamo'):
             protCoordinates = self._runTomoImportSetOfCoordinates('*.tbl', 'DYNAMO', 'TBL')
             output = getattr(protCoordinates, 'outputCoordinates', None)
@@ -178,13 +195,14 @@ class TestTomoImportSetOfCoordinates3D(BaseTest):
     def assertCoordinates(self, coordSet, size):
         self.assertTrue(coordSet,
                         "There was a problem with coordinates 3d output")
-        self.assertSetSize(coordSet,size=size)
+        self.assertSetSize(coordSet, size=size)
         self.assertTrue(coordSet.getBoxSize() == 32)
         self.assertTrue(coordSet.getSamplingRate() == 5.5)
 
 
 class TestTomoImportTomograms(BaseTest):
     """This class check if the protocol to import tomograms works properly."""
+
     @classmethod
     def setUpClass(cls):
         setupTestProject(cls)
@@ -202,6 +220,15 @@ class TestTomoImportTomograms(BaseTest):
         self.launchProtocol(protImport)
         return protImport
 
+    def _runImportTomograms2(self):
+        protImport = self.newProtocol(
+            tomo.protocols.ProtImportTomograms,
+            filesPath=self.tomogram,
+            filesPattern='',
+            samplingRate=1.35)
+        self.launchProtocol(protImport)
+        return protImport
+
     def test_importTomograms(self):
         protImport = self._runImportTomograms()
         output = getattr(protImport, 'outputTomograms', None)
@@ -209,17 +236,35 @@ class TestTomoImportTomograms(BaseTest):
                              "There was a problem with Import Tomograms protocol")
 
         for tomogram in protImport.outputTomograms.iterItems():
-
             self.assertTrue(tomogram.getXDim() == 1024,
-                                "There was a problem with Import Tomograms protocol")
+                            "There was a problem with Import Tomograms protocol")
             self.assertIsNotNone(tomogram.getYDim() == 1024,
-                                "There was a problem with Import Tomograms protocol")
+                                 "There was a problem with Import Tomograms protocol")
 
-            self.assertTrue(tomogram.getAcquisition().getAngleMax() == 40, "There was a problem with the acquisition angle max")
-            self.assertTrue(tomogram.getAcquisition().getAngleMin() == -40, "There was a problem with the acquisition angle min")
+            self.assertTrue(tomogram.getAcquisition().getAngleMax() == 40,
+                            "There was a problem with the acquisition angle max")
+            self.assertTrue(tomogram.getAcquisition().getAngleMin() == -40,
+                            "There was a problem with the acquisition angle min")
 
             break
 
+        protImport2 = self._runImportTomograms2()
+        output2 = getattr(protImport2, 'outputTomograms', None)
+        self.assertIsNotNone(output2,
+                             "There was a problem with Import Tomograms protocol")
+
+        for tomogram in protImport2.outputTomograms.iterItems():
+            self.assertTrue(tomogram.getXDim() == 1024,
+                            "There was a problem with Import Tomograms protocol")
+            self.assertIsNotNone(tomogram.getYDim() == 1024,
+                                 "There was a problem with Import Tomograms protocol")
+
+            self.assertTrue(tomogram.getAcquisition().getAngleMax() == 60,
+                            "There was a problem with the acquisition angle max")
+            self.assertTrue(tomogram.getAcquisition().getAngleMin() == -60,
+                            "There was a problem with the acquisition angle min")
+
+            break
 
 class TestTomoBaseProtocols(BaseTest):
     @classmethod
@@ -238,9 +283,11 @@ class TestTomoBaseProtocols(BaseTest):
             magnification=105000,
             sphericalAberration=2.7,
             amplitudeContrast=0.1,
-            samplingRate=1.35,
+            samplingRate=0.675,
             doseInitial=0,
-            dosePerFrame=0.3)
+            dosePerFrame=0.375,
+            tiltAxisAngle=84.1
+        )
         self.launchProtocol(protImport)
         return protImport
 
@@ -264,58 +311,61 @@ class TestTomoBaseProtocols(BaseTest):
 
 
 class TestTomoImportTsFromPattern(BaseTest):
-        @classmethod
-        def setUpClass(cls):
-            setupTestProject(cls)
-            cls.dataset = DataSet.getDataSet('tomo-em')
-            cls.getFile = cls.dataset.getFile('etomo')
-            cls.getFileM = cls.dataset.getFile('empiar')
+    @classmethod
+    def setUpClass(cls):
+        setupTestProject(cls)
+        cls.dataset = DataSet.getDataSet('tomo-em')
+        cls.getFile = cls.dataset.getFile('etomo')
+        cls.getFileM = cls.dataset.getFile('empiar')
 
-        def _runImportTiltSeriesM(self, filesPattern='{TS}_{TO}_{TA}.mrc'):
-            protImport = self.newProtocol(
-                tomo.protocols.ProtImportTsMovies,
-                filesPath=self.getFileM,
-                filesPattern=filesPattern,
-                voltage=300,
-                magnification=105000,
-                sphericalAberration=2.7,
-                amplitudeContrast=0.1,
-                samplingRate=1.35,
-                doseInitial=0,
-                dosePerFrame=0.3)
-            self.launchProtocol(protImport)
-            return protImport
+    def _runImportTiltSeriesM(self, filesPattern='{TS}_{TO}_{TA}.mrc'):
+        protImport = self.newProtocol(
+            tomo.protocols.ProtImportTsMovies,
+            filesPath=self.getFileM,
+            filesPattern=filesPattern,
+            voltage=300,
+            magnification=105000,
+            sphericalAberration=2.7,
+            amplitudeContrast=0.1,
+            samplingRate=0.675,
+            doseInitial=0,
+            dosePerFrame=0.375,
+            tiltAxisAngle=84.1
+        )
+        self.launchProtocol(protImport)
+        return protImport
 
-        def _runImportTiltSeries(self):
-            protImport = self.newProtocol(
-                tomo.protocols.ProtImportTs,
-                filesPath=self.getFile,
-                filesPattern='BB{TS}.st',
-                minAngle=-55,
-                maxAngle=65,
-                stepAngle=2,
-                voltage=300,
-                magnification=105000,
-                sphericalAberration=2.7,
-                amplitudeContrast=0.1,
-                samplingRate=1.35,
-                doseInitial=0,
-                dosePerFrame=0.3)
-            self.launchProtocol(protImport)
-            return protImport
+    def _runImportTiltSeries(self):
+        protImport = self.newProtocol(
+            tomo.protocols.ProtImportTs,
+            filesPath=self.getFile,
+            filesPattern='BB{TS}.st',
+            minAngle=-55,
+            maxAngle=65,
+            stepAngle=2,
+            voltage=300,
+            magnification=105000,
+            sphericalAberration=2.7,
+            amplitudeContrast=0.1,
+            samplingRate=1.35,
+            doseInitial=0,
+            dosePerFrame=0.3,
+            tiltAxisAngle=84.1)
+        self.launchProtocol(protImport)
+        return protImport
 
-        def test_importTiltSeriesM(self):
-            protImport = self._runImportTiltSeriesM()
-            self.checkTSSet(protImport.outputTiltSeriesM, 2, 3)
+    def test_importTiltSeriesM(self):
+        protImport = self._runImportTiltSeriesM()
+        self.checkTSSet(protImport.outputTiltSeriesM, 2, 3)
 
-            return protImport
+        return protImport
 
-        def test_importTiltSeries(self):
-            protImport = self._runImportTiltSeries()
-            self.checkTSSet(protImport.outputTiltSeries, 2, 61, checkIds=True)
+    def test_importTiltSeries(self):
+        protImport = self._runImportTiltSeries()
+        self.checkTSSet(protImport.outputTiltSeries, 2, 61, checkIds=True)
 
-        def checkTSSet(self, set, size, anglesCount, checkIds=False):
-            """
+    def checkTSSet(self, set, size, anglesCount, checkIds=False):
+        """
             Check basic attributes of a TS set
 
             :param set: TiltSeries set (Movies or Images)
@@ -325,31 +375,33 @@ class TestTomoImportTsFromPattern(BaseTest):
             :return: None
             """
 
-            self.assertSetSize(set, size)
-            for ts in set:
-                self.assertEquals(ts.getSize(), anglesCount, "Size of tilt images is wrong.")
-                for i, ti in enumerate(ts):
-                    self.assertFalse(os.path.isabs(ti.getFileName()),
-                                     "Tilt image file %s is not absolute!. Should be relative.")
-                    self.assertTrue(os.path.islink(ti.getFileName()),
-                                    "Tilt series file %s is not a link." % ti.getFileName())
+        self.assertSetSize(set, size)
+        for ts in set:
+            self.assertEquals(ts.getSize(), anglesCount, "Size of tilt images is wrong.")
+            for i, ti in enumerate(ts):
+                self.assertFalse(os.path.isabs(ti.getFileName()),
+                                 "Tilt image file %s is not absolute!. Should be relative.")
+                self.assertTrue(os.path.islink(ti.getFileName()),
+                                "Tilt series file %s is not a link." % ti.getFileName())
+
+                # objId must start with 1 --> i +1
+                # When using {TO} the objId ends up with that value. For Movies it is 0, 39, 40
+                if checkIds:
+                    self.assertEqual(i + 1, ti.getObjId(), "Tilt image Movie objId is incorrect")
 
-                    # objId must start with 1 --> i +1
-                    # When using {TO} the objId ends up with that value. For Movies it is 0, 39, 40
-                    if checkIds:
-                        self.assertEqual(i+1, ti.getObjId(), "Tilt image Movie objId is incorrect")
 
 class TestTomoImportTsFromMdoc(BaseTest):
-
     VOLTAGE = 'voltage'
     MAGNIFICATION = 'magnification'
     SPH_ABERRATION = 'sphAberration'
     AMP_CONTRAST = 'ampContrast'
     DOSE_PER_FRAME = 'dosePerFrame'
+    ACCUM_DOSE = 'accumDose'
     PIXEL_SIZE = 'pixelSize'
     SIZE = 'size'
     FILENAME_LIST = 'filenames'
-    ACCUM_DOSE_LIST = 'doses'
+    INCOMING_DOSE_LIST = 'incDoses'
+    ACCUM_DOSE_LIST = 'accumDoses'
     ANGLE_LIST = 'angles'
     ACQ_ORDER_LIST = 'acqOrder'
 
@@ -371,11 +423,11 @@ class TestTomoImportTsFromMdoc(BaseTest):
             # Each slice will be a stack of frames, which means a different file
             pattern = 'SKvesicles_Pertuzumab_tomo%2.0f_%03d_%1.1f.mrc'
 
-            return [ pattern % (tomoNum, i, angle)
+            return [pattern % (tomoNum, i, angle)
                     for i, angle in enumerate(angles)]
         else:
             # Each slice will be referred to the same stack file, the one which represents the tilt series
-            return [ 'stack%2.0f.mrcs' % tomoNum] * len(angles)
+            return ['stack%2.0f.mrcs' % tomoNum] * len(angles)
 
     @classmethod
     def _genTestDict(cls, **kwargs):
@@ -385,26 +437,54 @@ class TestTomoImportTsFromMdoc(BaseTest):
             cls.SPH_ABERRATION: kwargs.get(cls.SPH_ABERRATION, None),
             cls.AMP_CONTRAST: kwargs.get(cls.AMP_CONTRAST, None),
             cls.DOSE_PER_FRAME: kwargs.get(cls.DOSE_PER_FRAME, None),
+            cls.ACCUM_DOSE: kwargs.get(cls.ACCUM_DOSE, None),
             cls.PIXEL_SIZE: kwargs.get(cls.PIXEL_SIZE, None),
             cls.SIZE: kwargs.get(cls.SIZE, None),
             cls.FILENAME_LIST: kwargs.get(cls.FILENAME_LIST, None),
+            cls.INCOMING_DOSE_LIST: kwargs.get(cls.INCOMING_DOSE_LIST, None),
             cls.ACCUM_DOSE_LIST: kwargs.get(cls.ACCUM_DOSE_LIST, None),
             cls.ANGLE_LIST: kwargs.get(cls.ANGLE_LIST, None),
             cls.ACQ_ORDER_LIST: kwargs.get(cls.ACQ_ORDER_LIST, None)
         }
 
+    def sortTestDataByAngle(self, testDataDict):
+
+        for key in testDataDict:
+            testData = testDataDict[key]
+            ind = numpy.argsort(testData[self.ANGLE_LIST])
+
+            incDoses = []
+            accumDoses = []
+            newAcqOrders = []
+            newAngles = []
+            newFiles = []
+            for index in ind:
+                newAcqOrders.append(testData[self.ACQ_ORDER_LIST][index])
+                incDoses.append(testData[self.INCOMING_DOSE_LIST][index])
+                accumDoses.append(testData[self.ACCUM_DOSE_LIST][index])
+                newAngles.append(testData[self.ANGLE_LIST][index])
+                newFiles.append(testData[self.FILENAME_LIST][index])
+
+            testData[self.ANGLE_LIST] = newAngles
+            testData[self.INCOMING_DOSE_LIST] = incDoses
+            testData[self.ACCUM_DOSE_LIST] = accumDoses
+            testData[self.ACQ_ORDER_LIST] = newAcqOrders
+
     def _genTestData(self, isTsMovie):
-        return {
+
+        testData = {
             'stack31': self._genTestDict(
                 voltage=300,
                 magnification=53000,
                 sphAberration=2.7,
                 ampContrast=0.1,
-                dosePerFrame=0.0891,
+                dosePerFrame=2.2096,
+                accumDose=11.0479,
                 pixelSize=self.sRate,
                 size=5,
                 filenames=self._getListOfFileNames(tomoNum=31, isTsMovie=isTsMovie),
-                doses=[0.0424, 0.2121, 0.2546, 0.3394, 0.4455],
+                incDoses=[2.2088, 2.1975, 2.2245, 2.2275, 2.1895],
+                accumDoses=[2.2088, 4.4063, 6.6308, 8.8583, 11.0479],
                 angles=[0.0036, 2.9683, -3.0250, -6.0251, 5.9684],
                 acqOrder=[1, 2, 3, 4, 5]
             ),
@@ -413,16 +493,23 @@ class TestTomoImportTsFromMdoc(BaseTest):
                 magnification=53000,
                 sphAberration=2.7,
                 ampContrast=0.1,
-                dosePerFrame=0.0806,
+                dosePerFrame=2.3406,
+                accumDose=11.7028,
                 pixelSize=self.sRate,
                 size=5,
                 filenames=self._getListOfFileNames(tomoNum=10, isTsMovie=isTsMovie),
-                doses=[0.1909, 0.2333, 0.2546, 0.2970, 0.4030],
+                incDoses=[2.3443, 2.3421, 2.3425, 2.3373, 2.3366],
+                accumDoses=[2.3443, 4.6864, 7.0289, 9.3662, 11.7028],
                 angles=[0.0026, 2.9708, -3.0255, -6.0241, 5.9684],
                 acqOrder=[1, 2, 3, 4, 5]
             )
         }
 
+        if not isTsMovie:
+            self.sortTestDataByAngle(testData)
+
+        return testData
+
     def _runImportTiltSeries(self, isTsMovie=False):
         prot = tomo.protocols.ProtImportTsMovies if isTsMovie else tomo.protocols.ProtImportTs
         protImport = self.newProtocol(
@@ -439,6 +526,7 @@ class TestTomoImportTsFromMdoc(BaseTest):
         self.assertSetSize(outputSet, size=2)
         self.assertAlmostEqual(outputSet.getSamplingRate(), self.sRate, delta=0.001)
         self.assertEqual(outputSet.getDimensions(), dimensions)
+
         # Check tilt series movies
         for tsM in outputSet:
             testDataDict = testDict[tsM.getTsId()]
@@ -450,22 +538,34 @@ class TestTomoImportTsFromMdoc(BaseTest):
             self.assertAlmostEqual(acq.getSphericalAberration(), testDataDict[self.SPH_ABERRATION], delta=0.01)
             self.assertAlmostEqual(acq.getAmplitudeContrast(), testDataDict[self.AMP_CONTRAST], delta=0.001)
             self.assertAlmostEqual(acq.getDosePerFrame(), testDataDict[self.DOSE_PER_FRAME], delta=0.0001)
+            self.assertAlmostEqual(acq.getAccumDose(), testDataDict[self.ACCUM_DOSE], delta=0.0001)
             # Check angles and accumulated dose per angular acquisition (tilt series image)
             filesList = testDataDict[self.FILENAME_LIST]
+            incDoseList = testDataDict[self.INCOMING_DOSE_LIST]
             accumDoseList = testDataDict[self.ACCUM_DOSE_LIST]
             angleList = testDataDict[self.ANGLE_LIST]
             acqOrderList = testDataDict[self.ACQ_ORDER_LIST]
+
+            previousAngle = None
+
             for i, tiM in enumerate(tsM):
                 self.assertEqual(tiM.getFileName(), prot._getExtraPath(filesList[i]))
                 self.assertTrue(os.path.islink(tiM.getFileName()),
                                 "Tilt series file %s is not a link." % tiM.getFileName())
-                self.assertAlmostEqual(tiM.getAcquisition().getDosePerFrame(), accumDoseList[i], delta=0.0001)
+                self.assertAlmostEqual(tiM.getAcquisition().getDosePerFrame(), incDoseList[i], delta=0.0001)
+                self.assertAlmostEqual(tiM.getAcquisition().getAccumDose(), accumDoseList[i], delta=0.0001)
                 self.assertAlmostEqual(tiM.getTiltAngle(), angleList[i], delta=0.0001)
                 # objId must start with 1 --> i +1
-                self.assertEqual(i+1, tiM.getObjId(), "Tilt image Movie objId is incorrect")
+                self.assertEqual(i + 1, tiM.getObjId(), "Tilt image Movie objId is incorrect")
                 # Acquisition order
                 self.assertEqual(tiM.getAcquisitionOrder(), acqOrderList[i])
 
+                if not isTsMovie:
+                    self.assertEqual(i+1, tiM.getIndex())
+                    if previousAngle is not None:
+                        self.assertTrue(previousAngle < tiM.getTiltAngle(), "Tilt images are not sorted by angle.")
+                    previousAngle = tiM.getTiltAngle()
+
     def test_importTiltSeriesM(self):
         isTsMovie = True
         # Run protocol
@@ -500,8 +600,8 @@ class TestTomoImportTsFromMdoc(BaseTest):
         ]
         VOLTAGE = '*Voltage*'
         expectedErrorKeyWordList = [
-            VOLTAGE,                      # Missing voltage
-            VOLTAGE                       # Missing voltage
+            VOLTAGE,  # Missing voltage
+            VOLTAGE  # Missing voltage
         ]
         self._checkMDocParsingErrorMsg(mdocList, expectedErrorKeyWordList)
 
@@ -511,11 +611,13 @@ class TestTomoImportTsFromMdoc(BaseTest):
         simErrorMdocDir = self.dataset.getFile('simErrorMdocDir')
         mdocList = [
             join(simErrorMdocDir, dataSet['noMaginficationMdoc']),
-            join(simErrorMdocDir, dataSet['noSamplingRateMdoc'])
+            join(simErrorMdocDir, dataSet['noSamplingRateMdoc']),
+            join(simErrorMdocDir, dataSet['noDoseMdoc'])
         ]
         expectedErrorKeyWordList = [
-            '*Magnification*',          # Missing Magnification
-            '*PixelSpacing*'            # Missing Sampling Rate
+            '*Magnification*',  # Missing Magnification
+            '*PixelSpacing*',  # Missing Sampling Rate
+            '*dose*'  # Not able to get the dose
         ]
         self._checkMDocParsingErrorMsg(mdocList, expectedErrorKeyWordList)
 
@@ -529,8 +631,8 @@ class TestTomoImportTsFromMdoc(BaseTest):
             join(simErrorMdocDir, dataSet['someMissingAnglesMdoc'])
         ]
         expectedErrorKeyWordList = [
-            ['*Voltage*', '*PixelSpacing*'],   # Missing voltage and sampling rate
-            '*TiltAngle*: 1 7 48'              # Missing tilt angles in slices 1, 7 and 48
+            ['*Voltage*', '*PixelSpacing*'],  # Missing voltage and sampling rate
+            '*TiltAngle*: 1 7 48'  # Missing tilt angles in slices 1, 7 and 48
         ]
         self._checkMDocParsingErrorMsg(mdocList, expectedErrorKeyWordList)
 
@@ -549,4 +651,3 @@ class TestTomoImportTsFromMdoc(BaseTest):
                     self.assertTrue(errorKeyword in errorMsg)
             else:
                 self.assertTrue(not errorMsg)
-
