@@ -24,11 +24,9 @@
 # *
 # **************************************************************************
 
-import glob
 import numpy as np
 import os
 import random
-from os.path import join
 
 import pyworkflow.utils as pwutils
 import pwem.protocols as emprot
@@ -40,9 +38,9 @@ from . import DataSet
 from ..objects import SetOfTiltSeriesM, SetOfTiltSeries, Coordinate3D, Tomogram
 from ..utils import existsPlugin
 import tomo.protocols
-from tomo.protocols.protocol_ts_import import MDoc
 import tomo.constants as const
 
+
 class TestTomoBase(BaseTest):
     @classmethod
     def setUpClass(cls):
@@ -140,9 +138,11 @@ class TestTomoSubSetsTs(BaseTest):
             magnification=105000,
             sphericalAberration=2.7,
             amplitudeContrast=0.1,
-            samplingRate=1.35,
+            samplingRate=0.675,
             doseInitial=0,
-            dosePerFrame=0.3)
+            dosePerFrame=0.375,
+            tiltAxisAngle=84.7
+        )
         self.launchProtocol(protImport)
         return protImport
 
@@ -160,7 +160,9 @@ class TestTomoSubSetsTs(BaseTest):
             amplitudeContrast=0.1,
             samplingRate=1.35,
             doseInitial=0,
-            dosePerFrame=0.3)
+            dosePerFrame=0.3,
+            tiltAxisAngle=84.7
+        )
         self.launchProtocol(protImport)
         return protImport
 
@@ -197,9 +199,9 @@ class TestTomoSubSetsTs(BaseTest):
         # Create a subset 1 of TiltSeriesM
         print(pwutils.magentaStr("\n==> Create the subset 1 of TiltSeriesM"))
         tsSubset1 = self.newProtocol(emprot.ProtSubSet,
-                                       objLabel='subset 1',
-                                       chooseAtRandom=True,
-                                       nElements=1)
+                                     objLabel='subset 1',
+                                     chooseAtRandom=True,
+                                     nElements=1)
 
         tsSubset1.inputFullSet.set(protImport.outputTiltSeriesM)
         self.launchProtocol(tsSubset1)
@@ -207,9 +209,11 @@ class TestTomoSubSetsTs(BaseTest):
         # Create a subset subset 2 of TiltSeriesM
         print(pwutils.magentaStr("\n==> Create the subset 2 of TiltSeriesM"))
         tsSubset2 = self.newProtocol(emprot.ProtSubSet,
-                                       objLabel='subset 2',
-                                       chooseAtRandom=True,
-                                       nElements=1)
+                                     objLabel='subset 2',
+                                     chooseAtRandom=False,
+                                     inputSubSet=tsSubset1.outputTiltSeriesM,
+                                     setOperation=1,
+                                     nElements=1)
 
         tsSubset2.inputFullSet.set(protImport.outputTiltSeriesM)
         self.launchProtocol(tsSubset2)
@@ -217,9 +221,9 @@ class TestTomoSubSetsTs(BaseTest):
         # Create a subset subset 3 of TiltSeriesM
         print(pwutils.magentaStr("\n==> Create the subset 3 of TiltSeriesM"))
         tsSubset3 = self.newProtocol(emprot.ProtSubSet,
-                                       objLabel='subset 3',
-                                       chooseAtRandom=True,
-                                       nElements=2)
+                                     objLabel='subset 3',
+                                     chooseAtRandom=True,
+                                     nElements=2)
 
         tsSubset3.inputFullSet.set(protImport.outputTiltSeriesM)
         self.launchProtocol(tsSubset3)
@@ -227,8 +231,8 @@ class TestTomoSubSetsTs(BaseTest):
         # create merge protocol
         print(pwutils.magentaStr("\n==> Join subsets 1 and 2 "))
         joinTsSets = self.newProtocol(emprot.ProtUnionSet,
-                                       objLabel='join TiltSeriesM sets',
-                                       ignoreExtraAttributes=True)
+                                      objLabel='join TiltSeriesM sets',
+                                      ignoreExtraAttributes=True)
         joinTsSets.inputSets.append(tsSubset1.outputTiltSeriesM)
         joinTsSets.inputSets.append(tsSubset2.outputTiltSeriesM)
         joinTsSets.ignoreDuplicates.set(False)
@@ -248,7 +252,7 @@ class TestTomoSubSetsTs(BaseTest):
         print(pwutils.magentaStr(
             "\n==> Intersection subsets of TiltSeriesM"))
         label = '%s - %s,%s ' % (
-        tsSubset3.outputTiltSeriesM.getClassName(), 1, 2)
+            tsSubset3.outputTiltSeriesM.getClassName(), 1, 2)
         tsSubset = self.newProtocol(emprot.ProtSubSet)
         tsSubset.setObjLabel(label + 'intersection')
         tsSubset.inputFullSet.set(setFull)
@@ -335,9 +339,9 @@ class TestTomoSubSetsTomograms(BaseTest):
         # Create a subset with 1 tomograms
         print(pwutils.magentaStr("\n==> Create the subset 1 of Tomograms"))
         tomoSubset1 = self.newProtocol(emprot.ProtSubSet,
-                                      objLabel='subset 1',
-                                      chooseAtRandom=True,
-                                      nElements=1)
+                                       objLabel='subset 1',
+                                       chooseAtRandom=True,
+                                       nElements=1)
 
         tomoSubset1.inputFullSet.set(protImport.outputTomograms)
         self.launchProtocol(tomoSubset1)
@@ -346,7 +350,9 @@ class TestTomoSubSetsTomograms(BaseTest):
         print(pwutils.magentaStr("\n==> Create the subset 2 of Tomograms"))
         tomoSubset2 = self.newProtocol(emprot.ProtSubSet,
                                        objLabel='subset 2',
-                                       chooseAtRandom=True,
+                                       chooseAtRandom=False,
+                                       inputSubSet=tomoSubset1.outputTomograms,
+                                       setOperation=1,
                                        nElements=1)
 
         tomoSubset2.inputFullSet.set(protImport.outputTomograms)
@@ -365,8 +371,8 @@ class TestTomoSubSetsTomograms(BaseTest):
         # create merge protocol
         print(pwutils.magentaStr("\n==> Join subsets 1 and 2 "))
         joinTomoSet = self.newProtocol(emprot.ProtUnionSet,
-                                   objLabel='join Tomograms sets',
-                                   ignoreExtraAttributes=True)
+                                       objLabel='join Tomograms sets',
+                                       ignoreExtraAttributes=True)
         joinTomoSet.inputSets.append(tomoSubset1.outputTomograms)
         joinTomoSet.inputSets.append(tomoSubset2.outputTomograms)
         self.launchProtocol(joinTomoSet, wait=True)
@@ -399,9 +405,9 @@ class TestTomoSubSetsTomograms(BaseTest):
 
             # Check properties
             self.assertTrue(tomoSubset2.outputTomograms.equalAttributes(outputTomo,
-                                                 ignore=['_mapperPath',
-                                                         '_size'],
-                                                 verbose=True),
+                                                                        ignore=['_mapperPath',
+                                                                                '_size'],
+                                                                        verbose=True),
                             "Intersection subset attributes are wrong")
 
             for elem in outputTomo:
@@ -475,14 +481,14 @@ class TestTomoSubSetsSubTomograms(BaseTest):
         self.assertTrue(protImport.outputSubTomograms.getDim()[1] == 32)
         self.assertTrue(protImport.outputSubTomograms.getDim()[2] == 32)
         self.assertSetSize(protImport.outputSubTomograms, 4,
-                             "There was a problem with Import SubTomograms protocol")
+                           "There was a problem with Import SubTomograms protocol")
 
         # Create a subset with 1 tomograms
         print(pwutils.magentaStr("\n==> Create the subset 1 of SubTomograms"))
         tomoSubset1 = self.newProtocol(emprot.ProtSubSet,
-                                      objLabel='subset 1',
-                                      chooseAtRandom=True,
-                                      nElements=2)
+                                       objLabel='subset 1',
+                                       chooseAtRandom=True,
+                                       nElements=2)
 
         tomoSubset1.inputFullSet.set(protImport.outputSubTomograms)
         self.launchProtocol(tomoSubset1)
@@ -491,7 +497,9 @@ class TestTomoSubSetsSubTomograms(BaseTest):
         print(pwutils.magentaStr("\n==> Create the subset 2 of SubTomograms"))
         tomoSubset2 = self.newProtocol(emprot.ProtSubSet,
                                        objLabel='subset 2',
-                                       chooseAtRandom=True,
+                                       chooseAtRandom=False,
+                                       inputSubSet=tomoSubset1.outputSubTomograms,
+                                       setOperation=1,
                                        nElements=2)
 
         tomoSubset2.inputFullSet.set(protImport.outputSubTomograms)
@@ -510,8 +518,8 @@ class TestTomoSubSetsSubTomograms(BaseTest):
         # create merge protocol
         print(pwutils.magentaStr("\n==> Join subsets 1 and 2 "))
         joinTomoSet = self.newProtocol(emprot.ProtUnionSet,
-                                   objLabel='join SubTomograms sets',
-                                   ignoreExtraAttributes=True)
+                                       objLabel='join SubTomograms sets',
+                                       ignoreExtraAttributes=True)
         joinTomoSet.inputSets.append(tomoSubset1.outputSubTomograms)
         joinTomoSet.inputSets.append(tomoSubset2.outputSubTomograms)
         self.launchProtocol(joinTomoSet, wait=True)
@@ -544,9 +552,9 @@ class TestTomoSubSetsSubTomograms(BaseTest):
 
             # Check properties
             self.assertTrue(tomoSubset2.outputSubTomograms.equalAttributes(outputTomo,
-                                                 ignore=['_mapperPath',
-                                                         '_size'],
-                                                 verbose=True),
+                                                                           ignore=['_mapperPath',
+                                                                                   '_size'],
+                                                                           verbose=True),
                             "Intersection subset attributes are wrong")
 
             for elem in outputTomo:
@@ -555,71 +563,6 @@ class TestTomoSubSetsSubTomograms(BaseTest):
                                 'object id %s not in set: %s'
                                 % (elem.getObjId(), setSubIds))
 
-class TestTomoImportSetOfCoordinates3D(BaseTest):
-    """This class check if the protocol to import set of coordinates 3d works properly."""
-
-    @classmethod
-    def setUpClass(cls):
-        setupTestProject(cls)
-        cls.dataset = DataSet.getDataSet('tomo-em')
-        cls.tomogram = cls.dataset.getFile('tomo1')
-
-    def _runTomoImportSetOfCoordinates(self, pattern, program, ext):
-        protImportTomogram = self.newProtocol(tomo.protocols.ProtImportTomograms,
-                                              filesPath=self.tomogram,
-                                              samplingRate=5)
-
-        self.launchProtocol(protImportTomogram)
-
-        output = getattr(protImportTomogram, 'outputTomograms', None)
-
-        self.assertIsNotNone(output,
-                             "There was a problem with tomogram output")
-
-        protImportCoordinates3d = self.newProtocol(tomo.protocols.ProtImportCoordinates3D,
-                                 objLabel='Import from %s - %s' %(program, ext),
-                                 auto=tomo.protocols.ProtImportCoordinates3D.IMPORT_FROM_AUTO,
-                                 filesPath=self.dataset.getPath(),
-                                 importTomograms=protImportTomogram.outputTomograms,
-                                 filesPattern=pattern, boxSize=32,
-                                 samplingRate=5.5)
-        self.launchProtocol(protImportCoordinates3d)
-
-        return protImportCoordinates3d
-
-    def test_import_set_of_coordinates_3D(self):
-        outputs = []
-        if existsPlugin('emantomo'):
-            protCoordinates = self._runTomoImportSetOfCoordinates('*.json', 'EMAN', 'JSON')
-            output = getattr(protCoordinates, 'outputCoordinates', None)
-            self.assertTrue(output,
-                                 "There was a problem with coordinates 3d output")
-            self.assertTrue(output.getSize() == 19)
-            self.assertTrue(output.getBoxSize() == 32)
-            self.assertTrue(output.getSamplingRate() == 5.5)
-            outputs.append(output)
-
-        protCoordinates2 = self._runTomoImportSetOfCoordinates('*.txt', 'TOMO', 'TXT')
-        output2 = getattr(protCoordinates2, 'outputCoordinates', None)
-        self.assertTrue(output2,
-                             "There was a problem with coordinates 3d output")
-        self.assertTrue(output2.getSize() == 5)
-        self.assertTrue(output2.getBoxSize() == 32)
-        self.assertTrue(output2.getSamplingRate() == 5.5)
-        outputs.append(output2)
-
-        if existsPlugin('dynamo'):
-            protCoordinates2 = self._runTomoImportSetOfCoordinates('*.tbl', 'DYNAMO', 'TBL')
-            output3 = getattr(protCoordinates2, 'outputCoordinates', None)
-            self.assertTrue(output2,
-                                 "There was a problem with coordinates 3d output")
-            self.assertTrue(output2.getSize() == 5)
-            self.assertTrue(output2.getBoxSize() == 32)
-            self.assertTrue(output2.getSamplingRate() == 5.5)
-            outputs.append(output3)
-
-        return outputs
-
 
 class TestTomoPreprocessing(BaseTest):
     @classmethod
@@ -638,18 +581,18 @@ class TestTomoPreprocessing(BaseTest):
             magnification=105000,
             sphericalAberration=2.7,
             amplitudeContrast=0.1,
-            samplingRate=1.35,
+            samplingRate=0.675,
             doseInitial=0,
-            dosePerFrame=0.3)
+            dosePerFrame=0.375)
         self.launchProtocol(protImport)
         return protImport
 
     def test_preprocess1(self):
         try:
-            import gctf.protocols
-            import imod.protocols
+            import motioncorr.protocols
+            import cistem.protocols
         except ImportError:
-            print('To run this test scipion-em-gctf and scipion-em-imod plugins are required. '
+            print('To run this test scipion-em-cistem and scipion-em-motioncorr plugins are required. '
                   'This test will not be executed unless this plugins are installed')
             return
 
@@ -665,7 +608,7 @@ class TestTomoPreprocessing(BaseTest):
 
         # --------- Motion correction with motioncor2 for Tilt-series ------
         protMc = self.newProtocol(
-            tomo.protocols.ProtTsAverage,
+            motioncorr.protocols.ProtTsMotionCorr,
             inputTiltSeriesM=protImport.outputTiltSeriesM,
             binFactor=2.0,
             gpuList=gpuList,
@@ -673,27 +616,14 @@ class TestTomoPreprocessing(BaseTest):
         )
         self.launchProtocol(protMc)
 
-        # --------- CTF estimation with Gctf ------
-        protGctf = self.newProtocol(
-            gctf.protocols.ProtTsGctf,
+        # --------- CTF estimation with ctffind ------
+        protCtffind = self.newProtocol(
+            cistem.protocols.CistemProtTsCtffind,
             inputTiltSeries=protMc.outputTiltSeries,
-            gpuList=gpuList,
-            numberOfThreads=threads,
+            lowRes=50, minDefocus=30000,
+            numberOfThreads=threads
         )
-        self.launchProtocol(protGctf)
-
-        # # -------- Basic alignment and reconstruction with IMOD ------
-        # protImodAuto = self.newProtocol(
-        #     imod.protocols.ProtImodAuto3D,
-        #     inputTiltSeries=protGctf.outputTiltSeries,
-        #     excludeList=1,
-        #     rotationAngle=90,
-        #     zWidth=400,
-        #     useRaptor=True,
-        #     markersDiameter=20,
-        #     markersNumber=20
-        # )
-        # self.launchProtocol(protImodAuto)
+        self.launchProtocol(protCtffind)
 
 
 class TestTomoAssignAlignment(BaseTest):
@@ -711,8 +641,8 @@ class TestTomoAssignAlignment(BaseTest):
                                       samplingRate=5)
         self.launchProtocol(protImport)
         protImport2 = self.newProtocol(tomo.protocols.ProtImportSubTomograms,
-                                      filesPath=self.setOfSubtomograms,
-                                      samplingRate=5)
+                                       filesPath=self.setOfSubtomograms,
+                                       samplingRate=5)
         self.launchProtocol(protImport2)
         from xmipp2.protocols import Xmipp2ProtMLTomo
         protMltomo = self.newProtocol(Xmipp2ProtMLTomo,
@@ -723,16 +653,16 @@ class TestTomoAssignAlignment(BaseTest):
                                       angularSampling=30)
         self.launchProtocol(protMltomo)
         self.assertIsNotNone(protMltomo.outputSubtomograms,
-                         "There was a problem with the output")
+                             "There was a problem with the output")
         self.assertIsNotNone(protMltomo.outputClassesSubtomo,
-                         "There was a problem with the output")
+                             "There was a problem with the output")
         return protImport2, protMltomo
 
     def _assignAlignment(self):
         protImport2, protMltomo = self._runPreviousProtocols()
         assign = self.newProtocol(tomo.protocols.ProtAlignmentAssignSubtomo,
-                                 input=protImport2.outputSubTomograms,
-                                 inputAlignment=protMltomo.outputSubtomograms)
+                                  input=protImport2.outputSubTomograms,
+                                  inputAlignment=protMltomo.outputSubtomograms)
         self.launchProtocol(assign)
         self.assertIsNotNone(assign.outputAligned,
                              "There was a problem with the output")
@@ -756,23 +686,23 @@ class TestTomoAssignTomo2Subtomo(BaseTest):
 
     def _runPreviousProtocols(self):
         protImportTomo = self.newProtocol(tomo.protocols.ProtImportTomograms,
-                                        filesPath=self.setOfTomograms,
-                                        filesPattern='',
-                                        acquisitionAngleMax=40,
-                                        acquisitionAngleMin=-40,
-                                        samplingRate=1.35)
+                                          filesPath=self.setOfTomograms,
+                                          filesPattern='',
+                                          acquisitionAngleMax=40,
+                                          acquisitionAngleMin=-40,
+                                          samplingRate=1.35)
         self.launchProtocol(protImportTomo)
         protImportSubtomo = self.newProtocol(tomo.protocols.ProtImportSubTomograms,
-                                              filesPath=self.setOfSubtomograms,
-                                              samplingRate=5)
+                                             filesPath=self.setOfSubtomograms,
+                                             samplingRate=5)
         self.launchProtocol(protImportSubtomo)
         return protImportTomo, protImportSubtomo
 
     def _assignTomos2subtomos(self):
         protImportTomo, protImportSubtomo = self._runPreviousProtocols()
         tomo2subtomo = self.newProtocol(tomo.protocols.ProtAssignTomo2Subtomo,
-                                 inputSubtomos=protImportSubtomo.outputSubTomograms,
-                                 inputTomos=protImportTomo.outputTomograms)
+                                        inputSubtomos=protImportSubtomo.outputSubTomograms,
+                                        inputTomos=protImportTomo.outputTomograms)
         self.launchProtocol(tomo2subtomo)
         self.assertIsNotNone(tomo2subtomo.outputSubtomograms,
                              "There was a problem with subtomograms output")
@@ -797,11 +727,11 @@ class TestTomoSplitEvenOdd(BaseTest):
 
     def _splitTomoSet(self):
         protImportTomo = self.newProtocol(tomo.protocols.ProtImportTomograms,
-                                        filesPath=self.setOfTomograms,
-                                        filesPattern='',
-                                        acquisitionAngleMax=40,
-                                        acquisitionAngleMin=-40,
-                                        samplingRate=1.35)
+                                          filesPath=self.setOfTomograms,
+                                          filesPattern='',
+                                          acquisitionAngleMax=40,
+                                          acquisitionAngleMin=-40,
+                                          samplingRate=1.35)
         self.launchProtocol(protImportTomo)
         split = self.newProtocol(tomo.protocols.ProtSplitEvenOddTomoSet,
                                  inputSet=protImportTomo.outputTomograms)
@@ -814,8 +744,8 @@ class TestTomoSplitEvenOdd(BaseTest):
 
     def _splitSubtomoSet(self):
         protImportSubtomo = self.newProtocol(tomo.protocols.ProtImportSubTomograms,
-                                              filesPath=self.setOfSubtomograms,
-                                              samplingRate=5)
+                                             filesPath=self.setOfSubtomograms,
+                                             samplingRate=5)
         self.launchProtocol(protImportSubtomo)
         split = self.newProtocol(tomo.protocols.ProtSplitEvenOddTomoSet,
                                  inputSet=protImportSubtomo.outputSubTomograms)
@@ -852,6 +782,7 @@ class TestTomoSplitEvenOdd(BaseTest):
 
         return split
 
+
 class TestTomoCoordinatesOrigin(BaseTest):
 
     @classmethod
@@ -880,10 +811,10 @@ class TestTomoCoordinatesOrigin(BaseTest):
         self.coord.setMatrix(np.eye(4))
 
     def test_CenterGravity_Origin(self):
-        '''Test CENTER_GRAVITY convention (origin is placed in the center of
+        """Test CENTER_GRAVITY convention (origin is placed in the center of
         the Tomogram so Position (-dimx/2,-dimy/2,-dimz/2) should remain the same
         (i.e. we follow the default Scipion convention so coordinate is not transformed).
-        '''
+        """
         self._createCoordinate3D()
         x, y, z = self.Tomo.getDim()
         trPosition = self.coord.getPosition(const.CENTER_GRAVITY)
@@ -895,11 +826,11 @@ class TestTomoCoordinatesOrigin(BaseTest):
                          'CENTER_GRAVITY: Conversion did not return the expected value for Z axis')
 
     def test_BottomLeftCorner_Origin(self):
-        '''Test BOTTOM_LEFT_CORNER convention (origin is originally placed
+        """Test BOTTOM_LEFT_CORNER convention (origin is originally placed
         in the center of the Tomogram so Position (-dimx/2,-dimy/2,-dimz/2)
         should be converter into (0,0,0) (i.e. the bottom left corner should
         be now the origin).
-        '''
+        """
         self._createCoordinate3D()
         trPosition = self.coord.getPosition(const.BOTTOM_LEFT_CORNER)
         self.assertEqual(0, trPosition[0],
@@ -910,11 +841,11 @@ class TestTomoCoordinatesOrigin(BaseTest):
                          'BOTTOM_LEFT_CORNER: Conversion did not return the expected value for Z axis')
 
     def test_Scipion_Origin(self):
-        '''Test SCIPION convention (by default, origin is placed
+        """Test SCIPION convention (by default, origin is placed
         in the center of the Tomogram so Position (-dimx/2,-dimy/2,-dimz/2)
         should remain the same for this test (i.e. we follow the default Scipion convention
         so coordinate is not transformed).
-        '''
+        """
         self._createCoordinate3D()
         x, y, z = self.Tomo.getDim()
         trPosition = self.coord.getPosition(const.SCIPION)
@@ -925,5 +856,6 @@ class TestTomoCoordinatesOrigin(BaseTest):
         self.assertEqual(-0.5 * z, trPosition[2],
                          'SCIPION: Conversion did not return the expected value for Z axis')
 
+
 if __name__ == 'main':
     pass
