@@ -26,7 +26,10 @@
 
 import os
 
+import pyworkflow.utils as pwutils
+
 import pyworkflow.viewer as pwviewer
+from pwem.protocols import EMProtocol
 from pwem.viewers import ObjectView
 from pyworkflow.protocol import LabelParam
 
@@ -72,7 +75,7 @@ class TomoDataViewer(pwviewer.Viewer):
 
         elif issubclass(cls, tomo.objects.SetOfClassesSubTomograms):
             views.append(ClassesSubTomogramsView(self._project, obj.strId(),
-                                                   obj.getFileName()))
+                                                 obj.getFileName()))
 
         elif issubclass(cls, tomo.objects.SetOfMeshes):
             from .views_tkinter_tree import TomogramsTreeProvider, TomogramsDialog
@@ -86,33 +89,28 @@ class TomoDataViewer(pwviewer.Viewer):
 
         return views
 
+
 class TSMotionCorrectionViewer(pwviewer.ProtocolViewer):
     """ Wrapper to visualize outputs of tilt series motion correction protocols
     """
-
     _label = 'Tilt series motion correction viewer'
     _environments = [pwviewer.DESKTOP_TKINTER]
-    _targets = [
-        ProtTsCorrectMotion
-    ]
+    _targets = [ProtTsCorrectMotion]
 
     def _defineParams(self, form):
-            form.addSection(label='Visualization of tilt series')
-            form.addParam('displayFullTiltSeries', LabelParam,
-                          label='Display f*ull* frame aligned tilt series',
-                          help='Shows full frames aligned set of tilt series'
-                          )
-            if self.hasEvenSet():
-                form.addParam('displayEvenTiltSeries', LabelParam,
-                          label='Display *even* frames aligned tilt series',
-                          help='Shows even frames aligned set of tilt series'
-                          )
+        form.addSection(label='Visualization of tilt series')
+        form.addParam('displayFullTiltSeries', LabelParam,
+                      label='Display full frame aligned tilt series',
+                      help='Shows full frames aligned set of tilt series')
+        if self.hasEvenSet():
+            form.addParam('displayEvenTiltSeries', LabelParam,
+                          label='Display even frames aligned tilt series',
+                          help='Shows even frames aligned set of tilt series')
 
             if self.hasOddSet():
                 form.addParam('displayOddTiltSeries', LabelParam,
-                          label='Display *odd* frames aligned tilt series',
-                          help='Shows even frames aligned set of tilt series'
-                          )
+                              label='Display odd frames aligned tilt series',
+                              help='Shows even frames aligned set of tilt series')
 
     def hasEvenSet(self):
         return hasattr(self.protocol, SERIES_EVEN)
@@ -143,9 +141,64 @@ class TSMotionCorrectionViewer(pwviewer.ProtocolViewer):
         }
 
     def _visualize(self, setOfTiltSeries):
-
         from .views_tkinter_tree import TiltSeriesDialogView
         setTsView = TiltSeriesDialogView(self.getTkRoot(), self.protocol, setOfTiltSeries)
 
         return [setTsView]
 
+
+class CtfEstimationTomoViewer(pwviewer.Viewer):
+    """ This class implements a view using Tkinter CtfEstimationListDialog
+    and the CtfEstimationTreeProvider.
+    """
+    _label = 'CTF estimation viewer'
+    _environments = [pwviewer.DESKTOP_TKINTER]
+    _targets = [tomo.objects.SetOfCTFTomoSeries]
+
+    def __init__(self, parent, protocol, **kwargs):
+        pwviewer.Viewer.__init__(self, **kwargs)
+        self._tkParent = parent.root
+        self._protocol = protocol
+        self._title = 'CTF estimation viewer'
+
+    def plot1D(self, ctfSet, ctfId):
+        """ To be implemented in the viewers. """
+        return None
+
+    def plot2D(self, ctfSet, ctfId):
+        """ To be implemented in the viewers. """
+        return None
+
+    def visualize(self, obj, windows=None, protocol=None):
+        if not isinstance(obj, EMProtocol):
+            self.visualizeSet(obj)
+        else:
+            for name, output in self._protocol._iterOutputsNew():
+                if isinstance(output, tomo.objects.SetOfCTFTomoSeries):
+                    self.visualizeSet(output)
+
+    def visualizeSet(self, obj):
+        # JMRT: Local import to avoid importing Tkinter stuff at top level
+        from .views_tkinter_tree import (CtfEstimationTreeProvider,
+                                         CtfEstimationListDialog)
+        self._inputSetOfTiltSeries = obj.getSetOfTiltSeries()
+        self._provider = CtfEstimationTreeProvider(self._tkParent,
+                                                   self._protocol,
+                                                   obj)
+
+        CtfEstimationListDialog(self._tkParent, self._title, self._provider,
+                                self._protocol, self._inputSetOfTiltSeries,
+                                plot1Dfunc=self.getPlot1DCallback(),
+                                plot2Dfunc=self.getPlot2DCallback())
+
+    def getPlot1DCallback(self):
+        if not pwutils.isSameFunction(self.plot1D,
+                                      CtfEstimationTomoViewer.plot1D):
+            return self.plot1D
+        return None
+
+    def getPlot2DCallback(self):
+        if not pwutils.isSameFunction(self.plot2D,
+                                      CtfEstimationTomoViewer.plot2D):
+            return self.plot2D
+        return None
